/*
    terminatorX - realtime audio scratching software
    Copyright (C) 1999, 2000 Alexander Knig
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    File: tX_seqpar.h
 
    Description: Header to tX_seqpar.cc
*/    

#include <pthread.h>
#include <list>

#ifndef _tx_seqpar_h
#define _tx_seqpar_h 1

class tX_seqpar
{	
	protected:
	static list <tX_seqpar *> all;
	static list <tX_seqpar *> update;
	static pthread_mutex_t update_lock;
		
	int touched;
	int gui_active;
	void *vtt; /* have to make this void as tX_vtt.h includes this */
	
	unsigned int persistence_id;
	
	public:
	tX_seqpar(void *);
	tX_seqpar();
	~tX_seqpar();
	
	void set_vtt(void *);
	int is_touched() {return touched;}
	int is_untouched() {return !touched;}
	
	void touch() {touched=1;}
	void untouch() {touched=0;}
	
	int is_gui_active() { return gui_active; }
	
	static void create_persistence_ids();
	unsigned int get_persistence_id() {return persistence_id;}
	unsigned int set_persistence_id(int pid) {persistence_id=pid;}
	static tX_seqpar *get_sp_by_persistence_id(int pid);
	
	void receive_gui_value(const float value); /* slot for gui actions */
	void receive_input_value(const float value); /* slot for mouse 
							   keyboard and
							   MIDI actions */
	
	virtual void do_exec(const float value);
	virtual void exec_value(const float value);	
	virtual void do_update_graphics();
	void update_graphics();
	static void update_all_graphics();
};

class tX_seqpar_update : public tX_seqpar
{
	virtual void exec_value(const float value);	
};

class tX_seqpar_no_update : public tX_seqpar
{
	virtual void exec_value(const float value);	
};

/* now the classes for *real* sequenceable parameters */

class tX_seqpar_master_volume : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_master_pitch : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_speed : public tX_seqpar_no_update
{
	virtual void do_exec(const float value);
};

class tX_seqpar_vtt_volume : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_pitch : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_trigger : public tX_seqpar_no_update
{
	virtual void do_exec(const float value);
};

class tX_seqpar_vtt_loop : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_sync_client : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_sync_cycles : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_lp_enable : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_lp_gain : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_lp_reso : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_lp_freq : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_ec_enable : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_ec_length : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_ec_feedback : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
};

class tX_seqpar_vtt_mute : public tX_seqpar_no_update
{
	virtual void do_exec(const float value);
};
#endif 
